package com.hailify.sdk

import android.app.Activity
import android.content.Context
import android.location.Location
import android.util.Log
import android.widget.FrameLayout
import androidx.annotation.NonNull
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleOwner
import com.hailify.fleetsdk.DeliveryView
import com.hailify.fleetsdk.FleetSdk
import com.hailify.fleetsdk.listeners.CallListener
import com.hailify.fleetsdk.listeners.CompleteListener
import com.hailify.fleetsdk.listeners.NavigationListener
import com.hailify.fleetsdk.listeners.OrderStatusListener
import com.hailify.sdk.constants.MethodChannelNames

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** HelloPlugin */
class HailifyPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel: MethodChannel

  private lateinit var context: Context

  private var activity: Activity? = null

  private var deliveryView: DeliveryView? = null

  private var lifecycle: Lifecycle? = null

  private fun onAttachedToEngine(applicationContext: Context, messenger: BinaryMessenger) {
    context = applicationContext
    channel = MethodChannel(messenger, "hailify")
    channel.setMethodCallHandler(this)
  }

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    onAttachedToEngine(
      flutterPluginBinding.applicationContext,
      flutterPluginBinding.binaryMessenger
    );
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      MethodChannelNames.INITIALIZE -> {
        initialize(call, result)
      }
      MethodChannelNames.SET_DRIVER_ID -> {
        setDriverId(call, result)
      }
      MethodChannelNames.ENABLE_SANDBOX -> {
        enableSandbox(call, result)
      }
      MethodChannelNames.USE_SDK_LOCATION_ENGINE -> {
        useSDKLocationEngine(call, result)
      }
      MethodChannelNames.SET_LOCATION -> {
        setLocation(call, result)
      }
      MethodChannelNames.CANCEL_ORDER -> {
        cancelOrder(call, result)
      }
      MethodChannelNames.INITIALIZE_DELIVERY_VIEW -> {
        initializeDeliveryView()
      }
      MethodChannelNames.SET_CALL_LISTENER -> {
        setCallListener()
      }
      MethodChannelNames.SET_NAVIGATION_LISTENER -> {
        setNavigationListener()
      }
      MethodChannelNames.SET_ORDER_STATUS_LISTENER -> {
        setOrderStatusListener()
      }
      MethodChannelNames.SET_COMPLETE_LISTENER -> {
        setCompleteListener()
      }
      MethodChannelNames.SHOW_ORDER_POPUP -> {
        showOrderPopup()
      }
      MethodChannelNames.GET_ORDER_STATUS -> {
        getCurrentOrderStatus(result)
      }
      MethodChannelNames.SET_MAX_HEIGHT_PERCENTAGE -> {
        setMaxHeightPercentage(call, result)
      }
      else -> {
        result.notImplemented()
      }
    }
  }

  private fun initialize(call: MethodCall, result: Result) {
    val fleetName = call.argument<String>("fleetName") ?: ""
    val token = call.argument<String>("token") ?: ""
    FleetSdk.initilize(context, fleetName, token)
    FleetSdk.setPlatformType(4)
    Log.e("FleetModule", "initialize ===>$fleetName   $token")
    result.success(null)
  }

  private fun setDriverId(call: MethodCall, result: Result) {
    val driverId = call.argument<String>("driverId") ?: "";
    FleetSdk.setDriverID(driverId)
    Log.e("FleetModule", "setDriverId ===>$driverId")
    result.success(null)
  }

  private fun enableSandbox(call: MethodCall, result: Result) {
    val isEnableSandbox = call.argument<Boolean>("isEnableSandbox") ?: false
    FleetSdk.enableSandbox = isEnableSandbox
    Log.e("FleetModule", "isEnableSandbox ===>$isEnableSandbox")
    result.success(null)
  }

  private fun useSDKLocationEngine(call: MethodCall, result: Result) {
    val useSDKLocationEngine = call.argument<Boolean>("useSDKLocationEngine") ?: false
    FleetSdk.useSDKLocationEngine = useSDKLocationEngine
    Log.e("FleetModule", "useSDKLocationEngine ===>$useSDKLocationEngine")
    result.success(null)
  }

  private fun setLocation(call: MethodCall, result: Result) {
    val latitude = call.argument<Double>("latitude")
    val longitude = call.argument<Double>("longitude")
    val accuracy = call.argument<Double>("accuracy")
    val targetLocation = Location("")
    targetLocation.latitude = latitude ?: 0.0
    targetLocation.longitude = longitude ?: 0.0
    targetLocation.accuracy = (accuracy ?: 0.0).toFloat()
    Log.e("FleetModule", "latitude , longitude , accuracy  ===>$latitude $longitude $accuracy")
    FleetSdk.setLocation(targetLocation)
    result.success(null)
  }

  private fun cancelOrder(call: MethodCall, result: Result) {
    val additionalResponse = call.argument<String>("additionalResponse")
    FleetSdk.cancelOrder(additionalResponse ?: "")
    Log.e("FleetModule", "additionalResponse ===>$additionalResponse")
    result.success(null)
  }

  private fun initializeDeliveryView() {
    if (activity == null) {
      return
    }
    activity!!.runOnUiThread {
      var frameLayout: FrameLayout = activity!!.findViewById(android.R.id.content);
      deliveryView = DeliveryView(activity!!)
      deliveryView?.init(activity!!)
      val param = FrameLayout.LayoutParams(
        FrameLayout.LayoutParams.MATCH_PARENT,
        FrameLayout.LayoutParams.MATCH_PARENT
      )
      param.topMargin = 400
      deliveryView?.layoutParams = param
      frameLayout.addView(deliveryView)
      lifecycle?.addObserver(object : DefaultLifecycleObserver {
        override fun onCreate(owner: LifecycleOwner) {}
        override fun onStart(owner: LifecycleOwner) {}
        override fun onResume(owner: LifecycleOwner) {
          deliveryView?.onResume()
        }

        override fun onPause(owner: LifecycleOwner) {}
        override fun onStop(owner: LifecycleOwner) {}
        override fun onDestroy(owner: LifecycleOwner) {
          deliveryView?.onDestroy()
        }
      })
      channel.invokeMethod(MethodChannelNames.INITIALIZE_DELIVERY_VIEW_CALLBACK, null)
    }
  }

  private fun setCallListener() {
    if (deliveryView == null) {
      return
    }
    deliveryView?.setCallListener(object : CallListener {
      override fun callToLocation(phoneNumber: String) {
        val args = hashMapOf<String, String>()
        args["number"] = phoneNumber
        Log.e("FleetModule", "number ===>$phoneNumber")
        channel.invokeMethod(MethodChannelNames.SET_CALL_LISTENER_CALLBACK, args)
      }
    })
  }

  private fun setNavigationListener() {
    if (deliveryView == null) {
      return
    }
    deliveryView!!.setNavigationListener(object : NavigationListener {
      override fun navigateToLocation(v: Double, v1: Double) {
        val args = hashMapOf<String, Double>()
        args["latitude"] = v
        args["longitude"] = v1
        Log.e("FleetModule", "latitude longitude ===>$v $v1")
        channel.invokeMethod(MethodChannelNames.SET_NAVIGATION_LISTENER_CALLBACK, args)
      }
    })
  }

  private fun setOrderStatusListener() {
    if (deliveryView == null) {
      return
    }
    deliveryView!!.setOrderStatusListener(object : OrderStatusListener {
      override fun updatedOrderStatus(s: String, s1: String) {
        val args = hashMapOf<String, String>()
        args["status"] = s
        args["orderId"] = s1
        Log.e("FleetModule", "status orderId ===>$s $s1")
        channel.invokeMethod(MethodChannelNames.SET_ORDER_STATUS_LISTENER_CALLBACK, args)
      }
    })
  }

  private fun setCompleteListener() {
    if (deliveryView == null) {
      return
    }
    deliveryView!!.setCompleteListener(object : CompleteListener {
      override fun batchOrderCompleted() {
        val args = hashMapOf<String, String>()
        Log.e("FleetModule", "setCompleteListener")
        channel.invokeMethod(MethodChannelNames.SET_COMPLETE_LISTENER_CALLBACK, args)
      }
    })
  }

  private fun showOrderPopup() {
    if (deliveryView == null) {
      return
    }
    deliveryView!!.showOrderPopup()
  }

  private fun getCurrentOrderStatus(result: Result) {
    if (deliveryView == null) {
      return
    }
    try {
      val orderStatus = FleetSdk.getCurrentOrderStatus()
      result.success(orderStatus);
    } catch (e: Exception) {
      result.error(e.localizedMessage, e.message, e);
    }
  }

  private fun setMaxHeightPercentage(call: MethodCall, result: Result) {
    if (deliveryView == null) {
      return
    }
    val height = call.argument("height") as Int? ?: 70
    try {
      deliveryView?.maxHeightPercentage = height.toFloat()
      result.success("Max height set successfully");
    } catch (e: Exception) {
      result.error(e.localizedMessage, e.message, e);
    }
  }


  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity
    //lifecycle = FlutterLifecycleAdapter.getActivityLifecycle(binding)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    onDetachedFromActivity()
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    onAttachedToActivity(binding)
  }

  override fun onDetachedFromActivity() {
  }

}
